﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using CardFramework.Formats;
using CardFramework.Helpers;
using System.Xml.Serialization;
#endregion

#region "Class definitions"
namespace CardFramework.Applications.Fips201
{
  public class Chuid
  {
    #region "Members - Public"

    public Chuid()
    {
    }

    public static Chuid Parse(byte[] data)
    {
      Chuid result = new Chuid();
      TlvObject tlv;

      // FASC-N
      tlv = BerTlv.Query(data, @"53/30");
      result.FASCN = FASCN.Parse(tlv.Value);

      // GUID
      tlv = BerTlv.Query(data, @"53/34");
      result.GUID = tlv.Value;

      // Expiration Date
      tlv = BerTlv.Query(data, @"53/35");
      result.ExpirationDate = DateTime.MinValue;
      result.ExpirationDate = result.ExpirationDate.AddYears((tlv.Value[0] & 0x0F) * 1000);
      result.ExpirationDate = result.ExpirationDate.AddYears((tlv.Value[1] & 0x0F) * 100);
      result.ExpirationDate = result.ExpirationDate.AddYears((tlv.Value[2] & 0x0F) * 10);
      result.ExpirationDate = result.ExpirationDate.AddYears((tlv.Value[3] & 0x0F));
      result.ExpirationDate = result.ExpirationDate.AddMonths((tlv.Value[4] & 0x0F) * 10 );
      result.ExpirationDate = result.ExpirationDate.AddMonths((tlv.Value[5] & 0x0F));
      result.ExpirationDate = result.ExpirationDate.AddDays((tlv.Value[6] & 0x0F) * 10);
      result.ExpirationDate = result.ExpirationDate.AddDays((tlv.Value[7] & 0x0F));

      result.ExpirationDate = result.ExpirationDate.AddYears(-1); // To account for the min value being 1;
      result.ExpirationDate = result.ExpirationDate.AddMonths(-1);
      result.ExpirationDate = result.ExpirationDate.AddDays(-1);

      // Issuer Signature
      tlv = BerTlv.Query(data, @"53/3E");
      result.IssuerSignature = tlv.Value;

      return result;
    }


    public FASCN FASCN { get; set; }

    [XmlElement(DataType = "hexBinary")]
    public byte[] GUID { get; set; }
    
    public DateTime ExpirationDate { get; set; }

    [XmlElement(DataType = "hexBinary")]
    public byte[] IssuerSignature { get; set; }
    
    #endregion

    #region "Members - Private / Protected"

    #endregion

    #region "Constants / Declarations"
    #endregion
  }
}
#endregion