﻿#region "Copyright"
/******************************************************************************************
 Copyright (C) 2012 Kim O'Sullivan (kim@makina.com.au)

 Permission is hereby granted, free of charge, to any person obtaining a copy of 
 this software and associated documentation files (the "Software"), to deal in the 
 Software without restriction, including without limitation the rights to use, copy, 
 modify, merge, publish, distribute, sublicense, and/or sell copies of the Software, 
 and to permit persons to whom the Software is furnished to do so, subject to the 
 following conditions:
 
 The above copyright notice and this permission notice shall be included in all copies 
 or substantial portions of the Software.
 
 THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, 
 INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR 
 PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE 
 LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT 
 OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
 OTHER DEALINGS IN THE SOFTWARE.
******************************************************************************************/
#endregion

#region "Namespace definitions"
using System;
using System.Collections.Generic;
using System.Text;
using CardFramework.Protocols.Iso7816;
using CardFramework.Applications.Iso7816;
using System.IO;
using System.Threading;
#endregion

#region "Class definitions"
namespace CardFramework.Applications.Fips201
{
	public class Fips201Application : Iso7816Application
	{
		#region "Members - Public"

    public override bool Discover(Card card)
    {
      // Create a backup of the reference to our old card
      Card oldCard = this.Card;

      // Temporarily set the card
      this.Card = card;

      try
      {
        SelectApplication();
        return true;
      }
      catch (Exception)
      {
        return false;
      }
      finally
      {
        this.Card = oldCard;    
      }
    }

    public AptObject SelectApplication()
    {
      byte[] response = SelectFile(AID, SelectMode.DfName);
      AptObject apt = new AptObject(response);
      return apt;
    }

    public Chuid ReadChuid()
    {
      byte[] response = GetDataExtended(OBJ_CHUID, Iso7816Application.GetDataP1CurrentDF, Iso7816Application.GetDataP2CurrentDF);

      return Chuid.Parse(response);
    }
    
    #endregion


		#region "Members - Private / Protected"
		#endregion
	
		#region "Constants / Private Declarations"

    /// <summary>
    /// The PIV Application Identifier (AID)
    /// </summary>
    private readonly byte[] AID = { 0xA0, 0x00, 0x00, 0x03, 0x08, 0x00, 0x00, 0x10, 0x00, 0x01, 0x00 };

    private readonly byte[] OBJ_CCC = { 0x5C, 0x03, 0x5F, 0xC1, 0x07 };
    private readonly byte[] OBJ_CHUID = { 0x5C, 0x03, 0x5F, 0xC1, 0x02 };
    private readonly byte[] OBJ_X509_PIV = { 0x5C, 0x03, 0x5F, 0xC1, 0x05 };
    private readonly byte[] OBJ_FINGER = { 0x5C, 0x03, 0x5F, 0xC1, 0x03 };
    private readonly byte[] OBJ_SECURITY = { 0x5C, 0x03, 0x5F, 0xC1, 0x06 };
    private readonly byte[] OBJ_FACIAL = { 0x5C, 0x03, 0x5F, 0xC1, 0x08 };
    private readonly byte[] OBJ_PRINTED = { 0x5C, 0x03, 0x5F, 0xC1, 0x09 };
    private readonly byte[] OBJ_X509_DSIG = { 0x5C, 0x03, 0x5F, 0xC1, 0x0A };
    private readonly byte[] OBJ_X509_KEYMGMT = { 0x5C, 0x03, 0x5F, 0xC1, 0x0B };
    private readonly byte[] OBJ_X509_CARDAUTH = { 0x5C, 0x03, 0x5F, 0xC1, 0x01 };
    private readonly byte[] OBJ_DISCOVERY = { 0x5C, 0x01, 0x7E };

    #endregion
  }
}
#endregion