﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using PACManager.Controls;
using System.Reflection;

namespace PACManager
{
  public partial class FMain : Form
  {
    public FMain()
    {
      InitializeComponent();
    }

    public FMain(bool keyInjectionMode)
    {
      InitializeComponent();

      myKeyInjectionMode = keyInjectionMode;
    }

    private WizardScreen myActiveScreen;
    private bool myKeyInjectionMode = false;

    void myActiveScreen_RaiseNextButtonEvent(object sender, ScreenEventArgs e)
    {
      LoadScreen(e.Target);

      switch (e.Target.Name)
      {
        case "HomeScreen":
          panHeader.BackgroundImage = global::PACManager.Properties.Resources.Screen_Home;
          break;

        case "ConfigureScreen":
          panHeader.BackgroundImage = global::PACManager.Properties.Resources.Screen_Configure;
          break;

        case "ProcessScreen":
          panHeader.BackgroundImage = global::PACManager.Properties.Resources.Screen_Process;
          break;

        case "KeyInjectionScreen":
          panHeader.BackgroundImage = global::PACManager.Properties.Resources.Screen_Injection;
          break;
      }
    }

    void myActiveScreen_RaiseLastButtonEvent(object sender, ScreenEventArgs e)
    {
      LoadScreen(e.Target);
    }


    private void LoadScreen(WizardScreen screen)
    {
      Cursor = Cursors.WaitCursor;
      Application.DoEvents();

      try
      {
        // First cleanup any screen that is already there
        if (null != myActiveScreen)
        {
          // Remove from the main panel
          panBody.Controls.Remove(myActiveScreen);

          // Unhook the event
          myActiveScreen.OnNextButtonEvent -= new EventHandler<ScreenEventArgs>(myActiveScreen_RaiseNextButtonEvent);
          myActiveScreen.OnLastButtonEvent -= new EventHandler<ScreenEventArgs>(myActiveScreen_RaiseLastButtonEvent);

          // Shutdown
          myActiveScreen.Shutdown();
          myActiveScreen.Dispose();
          myActiveScreen = null;
        }

        // Are we loading a new screen?
        if (null != screen)
        {
          // Now position the next screen
          panBody.Controls.Add(screen);
          screen.Dock = DockStyle.Fill;

          // Now startup and show
          screen.Show();
          screen.Startup();

          // Set it as our new active screen & hook the events
          myActiveScreen = screen;
          myActiveScreen.OnNextButtonEvent += new EventHandler<ScreenEventArgs>(myActiveScreen_RaiseNextButtonEvent);
          myActiveScreen.OnLastButtonEvent += new EventHandler<ScreenEventArgs>(myActiveScreen_RaiseLastButtonEvent);
        }
      }
      catch (Exception)
      {
        MessageBox.Show("An error occurred changing screens.");
      }
      finally
      {
        Cursor = Cursors.Default;
      }

    }

    private void FMain_Load(object sender, EventArgs e)
    {
      // Retrieve the version info
      Assembly assem = Assembly.GetEntryAssembly();
      string msg = string.Format("{0} (version {1})", this.Text, assem.GetName().Version.ToString());
      this.Text = msg;

      // Check whether we should be loading into the Key Injection screen
      if (myKeyInjectionMode)
      {
        LoadScreen(new KeyInjectionScreen());
      }
      else
      {
        LoadScreen(new HomeScreen());
      }
    }

    private void FMain_FormClosing(object sender, FormClosingEventArgs e)
    {
      LoadScreen(null);
    }

    private void panBody_Paint(object sender, PaintEventArgs e)
    {

    }

  }
}
