﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using CardFramework.Helpers;
using System.Threading;

namespace PACManager
{
  public partial class FViewer : Form
  {
    public FViewer()
    {
      InitializeComponent();

    }

    public void LoadContext(CardContext context)
    {
      // Personalisation status
      if (context.Wiegand46 != null)
      {
        lblPLAID.Text = "Personalised";
      }
      else
      {
        lblPLAID.Text = "Initialised / Invalid";
      }

      if (context.VerifyMifare == true)
      {
        lblMifare.Text = "Personalised";
      }
      else
      {
        lblMifare.Text = "Factory conditions";
      }

      /*
       * GP/FIPS objects
       */
      lblICSerialNo.Text = context.CPLC.ICSerialNo.ToString();
      lblGUID.Text = Converters.ByteArrayToString(context.CHUID.GUID);
      lblPI.Text = context.CHUID.FASCN.PI;

      /*
       * OP MODES
       */
      if (context.Wiegand46 != null) lblOpMode1.Text = Converters.ByteArrayToString(context.Wiegand46);
      if (context.Wiegand26 != null) lblOpMode2.Text = Converters.ByteArrayToString(context.Wiegand26);
      if (!string.IsNullOrEmpty(context.PI)) lblOpMode3.Text = context.PI;
      if (context.UUID != null) lblOpMode4.Text = Converters.ByteArrayToString(context.UUID);
    }


    private void btnClose_Click(object sender, EventArgs e)
    {
      this.Close();
    }

    private void btnICSerialNo_Click(object sender, EventArgs e)
    {
      try
      {
        if (string.IsNullOrEmpty(lblICSerialNo.Text)) return;
        SetClipboardText(lblICSerialNo.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnGUID_Click(object sender, EventArgs e)  
    {
      try
      {
        if (string.IsNullOrEmpty(lblGUID.Text)) return;
        SetClipboardText(lblGUID.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnOpMode1_Click(object sender, EventArgs e)
    {
      try
      {
        if (string.IsNullOrEmpty(lblOpMode1.Text)) return;
        SetClipboardText(lblICSerialNo.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnOpMode2_Click(object sender, EventArgs e)
    {
      try
      {
        // Trim to 24 bits
        if (string.IsNullOrEmpty(lblOpMode2.Text)) return;
        uint w26 = Convert.ToUInt32(lblICSerialNo.Text) & 0xFFFFFF;
        SetClipboardText(w26.ToString());
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnOpMode3_Click(object sender, EventArgs e)
    {
      try
      {
        if (string.IsNullOrEmpty(lblOpMode3.Text)) return;
        SetClipboardText(lblOpMode3.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void btnOpMode4_Click(object sender, EventArgs e)
    {
      try
      {
        if (string.IsNullOrEmpty(lblOpMode4.Text)) return;
        SetClipboardText(lblOpMode4.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }
    }

    private void SetClipboardText(string text)
    {
      Thread t = new Thread(delegate()
      {
        try
        {
          Clipboard.SetText(text);
        }
        catch (Exception)
        {
          
        }
      });
      t.SetApartmentState(ApartmentState.STA);
      t.IsBackground = true;
      t.Start();
    }

    private void btnPI_Click(object sender, EventArgs e)
    {
      try
      {
        if (string.IsNullOrEmpty(lblPI.Text)) return;
        SetClipboardText(lblPI.Text);
        MessageBox.Show(@"Copied to clipboard", "Copy to clipboard", MessageBoxButtons.OK, MessageBoxIcon.Information);
      }
      catch (Exception)
      {
        MessageBox.Show(@"Failed to set the clipboard text", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
      }

    }
  }
}
