﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using CardFramework.Helpers;
using PACManager.Properties;
using CardFramework.Protocols.Mifare;

namespace PACManager
{
  public static class WiegandRecord
  {
    public const int UUIDLength = 16; // As per RFC-4122

    public static uint GenerateWiegand26(int value)
    {
      uint acl = 0;

      // Strip value to 24 bits
      acl = (uint)((value & 0x00FFFFFF) << 1);

      // Parity
      int oddParity = (Converters.Bitcount(acl) % 2 != 0) ? 1 : 0;
      int evenParity = (Converters.Bitcount(acl) % 2 == 0) ? 1 : 0;
      acl |= (uint)evenParity;
      acl |= (uint)oddParity << 25;

      return acl;
    }

    public static ulong GenerateWiegand46(int value)
    {
      value += Settings.Default.AclSeed;

      ulong acl = 0;

      // Card number
      acl |= (ulong)((ulong)value << 1);

      // Site Code
      acl |= (ulong)((ulong)Convert.ToUInt32(Settings.Default.AclSiteCode) << 25);

      // Parity
      ulong oddParity = (ulong)((Converters.Bitcount(acl) % 2 != 0) ? 1 : 0);
      ulong evenParity = (ulong)((Converters.Bitcount(acl) % 2 == 0) ? 1 : 0);
      acl |= evenParity;
      acl |= oddParity << 45;

      // Shift the value left for the padding
      acl <<= 18;
      acl = acl & 0x3FFFFFFFFFFFFFF;

      return acl;
    }
  }
}
