/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package plaidSam;

/**
 *
 * @author Administrator
 */
public class Tlv {

	/*
	 * CONSTANTS
	 */
	// Tag Class
	public static final byte TLV_CLASS_UNIVERSAL = (byte) 0x00;
	public static final byte TLV_CLASS_APPLICATION = (byte) 0x01;
	public static final byte TLV_CLASS_CONTEXT = (byte) 0x02;
	public static final byte TLV_CLASS_PRIVATE = (byte) 0x03;

	// Masks
	public static final byte MASK_CONSTRUCTED = (byte) 0x20;
	public static final byte MASK_LOW_TAG_NUMBER = (byte) 0x1F;
	public static final byte MASK_HIGH_TAG_NUMBER = (byte) 0x7F;
	public static final byte MASK_HIGH_TAG_MOREDATA = (byte) 0x80;
	public static final byte MASK_LONG_LENGTH = (byte) 0x80;
	public static final byte MASK_LENGTH = (byte) 0x7F;

	// Universal tags
	public static final byte ASN1_SEQUENCE = 0x10;
	public static final byte ASN1_STRING_OCTET = 0x04;

	public static final short TAG_NOT_FOUND = (short) 0;

	    public static short find(byte[] data, short offset, short tag)
	    {
	      while (offset < data.length)
	      {
	        // Is this our tag number?
	        if (tag == getTagNumber(data, offset))
	        {
	          // We've found it!
	          return offset;
	        }

	        // Is this a constructed object?
	        if (getIsConstructed(data, offset))
	        {
	          // Skip to the first child element
	          offset = getDataOffset(data, offset);
	        }
	        else
	        {
	          // Skip to the next
	          short length = getLength(data, offset);
	          offset = getDataOffset(data, offset);
	          offset += length;
	        }
	      }

	      return TAG_NOT_FOUND;
	    }

	    public static short findNext(byte[] data, short offset, short tag)
	    {
	      // First move to the next object

	      // Is this a constructed object?
	      if (getIsConstructed(data, offset))
	      {
	        // Skip to the first child element
	        offset = getDataOffset(data, offset);
	      }
	      else
	      {
	        // Skip to the next
	        short length = getLength(data, offset);
	        offset = getDataOffset(data, offset);
	        offset += length;
	      }

	      // Now find as normal
	      return find(data, offset, tag);
	    }

	    /*
	     * Returns whether the TLV data element at the current position is
	     * constructed or not (primitive).
	     */
	    public static boolean getIsConstructed(byte[] data, short offset)
	    {
	      // Check bit 6 of the T element
	      return ((data[offset] & MASK_CONSTRUCTED) != 0);
	    }
	    /*
	     * Returns the tag number for the TLV data element at the current position.
	     * NOTE: This does NOT include the class and/or constructed flag. NOTE: This
	     * only supports tag values up to 127.
	     */
	    public static byte getTagNumber(byte[] data, short offset)
	    {
	      if ((data[offset] & MASK_LOW_TAG_NUMBER) != MASK_LOW_TAG_NUMBER)
	      {
	        // This is a low-tag-number form (Tags 0 to 31)
	        return (byte)(data[offset] & MASK_LOW_TAG_NUMBER);
	      }
	      else
	      {
	        // This is a high-tag-number form (Tags 31+)
	        offset++;
	        if ((data[offset] & MASK_HIGH_TAG_MOREDATA)
	                == MASK_HIGH_TAG_MOREDATA)
	        {
	          // This implementation does not support tags > 127
	          //throw TlvException.getInstance(TlvException.TAG_NUMBER_EXCEEDS_127);
                    return 0;
	        }
	        else
	        {
	          return (byte)(data[offset] & MASK_HIGH_TAG_NUMBER);
	        }
	      }
	    }

	    public static short getLength(byte[] data, short offset)
	    {
	      // First, skip through the T element
	      if ((data[offset++] & MASK_LOW_TAG_NUMBER) == MASK_LOW_TAG_NUMBER)
	      {
	        // This is a high-tag-number form (Tags 31+)
	        if ((data[offset++] & MASK_HIGH_TAG_MOREDATA)
	                == MASK_HIGH_TAG_MOREDATA)
	        {
	          // This implementation does not support tags > 127
	          //throw TlvException.getInstance(TlvException.TAG_NUMBER_EXCEEDS_127);
	        }
	      }

	      // Is this a long-form length byte?
	      if ((data[offset] & MASK_LONG_LENGTH) == MASK_LONG_LENGTH)
	      {
	        // Is there more than 1 byte?
	        if ((data[offset] & MASK_LENGTH) != 1)
	        {
	          // We don't support multi-byte length definitions
	          //throw TlvException.getInstance(TlvException.TAG_LENGTH_EXCEEDS_127);
	        }
	        else
	        {
	          offset++;
	        }
	      }

              return (short)(data[offset] & (short)0xFF);
	    }

	    public static byte getClass(byte[] data, short offset)
	    {
	      return (byte)(data[offset] >> 6);
	    }

	    public static short getDataOffset(byte[] data, short offset)
	    {

	      // First, skip through the T element
	      if ((data[offset++] & MASK_LOW_TAG_NUMBER) == MASK_LOW_TAG_NUMBER)
	      {
	        // This is a high-tag-number form (Tags 31+)
	        if ((data[offset++] & MASK_HIGH_TAG_MOREDATA)
	                == MASK_HIGH_TAG_MOREDATA)
	        {
	          // This implementation does not support tags > 127
	          //throw TlvException.getInstance(TlvException.TAG_NUMBER_EXCEEDS_127);
	        }
	      }

	      // Is this a long-form length byte?
	      if ((data[offset] & MASK_LONG_LENGTH) == MASK_LONG_LENGTH)
	      {
	        // Is there more than 1 byte?
	        if ((data[offset] & MASK_LENGTH) != 1)
	        {
	          // We don't support multi-byte length definitions
	          //throw TlvException.getInstance(TlvException.TAG_LENGTH_EXCEEDS_127);
	        }
	        else
	        {
	          offset++;
	        }
	      }

	      return ++offset;
	    }

	    public static short toShort(byte[] data, short offset)
	    {
	      // Skip to the data value
	      offset = getDataOffset(data, offset);

	      return (short)((short)((data[offset++] << (short)8)) + (short)data[offset]);
	    }

	    public static byte toByte(byte[] data, short offset)
	    {

	      // Skip to the data value
	      offset = getDataOffset(data, offset);

	      return data[offset];
	    }

}